using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace TiledLevel
{
    class Blob : GameObject
    {
        private static TextureContainer image = new TextureContainer();

        private bool right;

        public Blob(Map map)
        {
            int x, y;

            base.texture = image;

            // The blob is 64x64 pixels, as our game tiles are 32x32 this makes him
            // 2x2 game tiles in size.
            size = new Vector2(2f, 2f);

            // Look for a valid position in the map to add the blob.
            // We will continually pick a random position until we find somewhere
            // that is empty.
            do
            {
                // Pick a random spot on the map
                x = 1 + Global.rnd.Next(map.Width - 2);
                y = 1 + Global.rnd.Next(map.Height - 2);
            }
            while (map.SolidSquare(x, y    ) || map.SolidSquare(x + 1, y    ) ||
                   map.SolidSquare(x, y + 1) || map.SolidSquare(x + 1, y + 1));

            // Set the position we've found.
            pos = new Vector2(x, y);
        }

        public override UpdateAction Update(Map map, List<GameObject> objects)
        {
            // Fix the X vector based on the flag indicating which direction we're going.
            if (right) vector.X = 0.06f;
            else vector.X = -0.06f;

            // Simulate gravity by adding a constant to the Y vector.
            vector.Y += 0.01f;

            // Move the blob.
            Direction dirn = Move(map);

            if ((dirn & Direction.Left) != 0 ||
                (dirn & Direction.Right) != 0)
            {
                // If the blob hit a wall to the left or right then change direction.
                right = !right;
            }

            if ((dirn & Direction.Down) != 0)
            {
                // If the blob is resting on a surface then jump occasionally.
                if (Global.rnd.Next(128) == 64)
                    vector.Y = -0.3f;
            }

            return base.Update(map, objects);
        }

        public override HitAction HitPlayer()
        {
            // Let the player object know they've been hurt.
            return HitAction.Damage;
        }

        public static void LoadContent(ContentManager content)
        {
            // Load the coin image.
            image.tex = content.Load<Texture2D>("gfx/blob");
        }
    }
}
