using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace TiledLevel
{
    class Coin : GameObject
    {
        private static TextureContainer image = new TextureContainer();

        public Coin(Map map)
        {
            int x, y;

            base.texture = image;

            // Look for a valid position in the map to add the coin.
            // We will continually pick a random position until we find somewhere that is a
            // space but is directly above a brick so it looks like the coin is resting on it.
            do
            {
                // Pick a random spot on the map
                x = 1 + Global.rnd.Next(map.Width - 2);
                y = 1 + Global.rnd.Next(map.Height - 2);
            }
            while (map.SolidSquare(x, y) || !map.SolidSquare(x, y + 1));

            // Set the position we've found.
            pos = new Vector2(x, y);

            // Render the coins behind the player and enemies.
            depth = 0.7f;
        }

        public override HitAction HitPlayer()
        {
            // Mark the coin for deletion when the player hits it so it appears
            // to get collected.
            flags |= Flags.Delete;

            return HitAction.None;
        }

        public static void LoadContent(ContentManager content)
        {
            // Load the coin image.
            image.tex = content.Load<Texture2D>("gfx/coin");
        }
    }
}
