using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;

namespace Shuggy
{
    class TileInfo
    {
        public const int SIZE = 32;

        private Texture2D tex;
        public Texture2D Tex
        {
            get { return tex; }
        }

        private bool masked;
        public bool Masked
        {
            get { return masked; }
        }

        private bool brick;     // In terms of edging bricks.
        public bool Brick
        {
            get { return brick; }
        }

        private string imgFile;
        private int collision;

        public TileInfo(string image, bool masked, int collision, bool brick)
        {
            this.imgFile = image;
            this.masked = masked;
            this.collision = collision;
            this.brick = brick;
        }

        public void LoadGraphicsContent(ContentManager content)
        {
            if (imgFile != null)
            {
                tex = content.Load<Texture2D>(imgFile);
                if ((tex.Width != TileInfo.SIZE || tex.Height != TileInfo.SIZE) &&
                    (tex.Width != TileInfo.SIZE / 4 || tex.Height != TileInfo.SIZE / 4))
                    throw new Exception(imgFile + " is the wrong size");
            }
        }
        /*
        public bool Solid(Vector2 point)
        {
            int bit = 0;

            if (point.Y >= 0.75f) bit += 12;
            else if (point.Y >= 0.5f) bit += 8;
            else if (point.Y >= 0.25f) bit += 4;

            if (point.X >= 0.75f) bit += 3;
            else if (point.X >= 0.5f) bit += 2;
            else if (point.X >= 0.25f) bit += 1;

            return ((collision & (1 << bit)) != 0);
        }
        */
        public bool Solid(int x, int y)
        {
            int bit = y * 4 + x;

            return ((collision & (1 << bit)) != 0);
        }

        public bool Solid()
        {
            return (collision != 0);
        }
    }

    static class TilesInfo
    {
        private static int TILE_COUNT = 62;

        public const int BRICK = 1;
        public const int BRICK_T = 2;
        public const int BRICK_R = 3;
        public const int BRICK_B = 4;
        public const int BRICK_L = 5;
        public const int BRICK_TL = 6;
        public const int BRICK_TR = 7;
        public const int BRICK_BL = 8;
        public const int BRICK_BR = 9;
        public const int BRICK_TLI = 10;
        public const int BRICK_TRI = 11;
        public const int BRICK_BLI = 12;
        public const int BRICK_BRI = 13;
        public const int BRICK_MINI = 14;
        public const int SPACE_MINI = 15;
        public const int MINI_BRICKS = 23;
        public const int MINI_BRICK_BASE = MINI_BRICKS - 1;
        public const int LAVA_FRAMES = 8;
        public const int BLACK = 37;
        public const int LAVA_0 = 38;
        public const int LAVATOP_0 = 46;
        public const int SPIKE = 54;
        public const int CAGED_GEM = 55;
        public const int CLEAR_SOLID = 56;
        public const int UPDRAFT = 57;
        public const int CARD_SPOT = 58;

        private static TileInfo[] tiles = new TileInfo[TILE_COUNT];

        static TilesInfo()
        {
            tiles[0] = new TileInfo(null, true, 0, false);

            tiles[BRICK] = new TileInfo("gfx/map/brick", false, 0xffff, true);
            tiles[BRICK_T] = new TileInfo("gfx/map/brickt", false, 0xffff, true);
            tiles[BRICK_R] = new TileInfo("gfx/map/brickr", false, 0xffff, true);
            tiles[BRICK_B] = new TileInfo("gfx/map/brickb", false, 0xffff, true);
            tiles[BRICK_L] = new TileInfo("gfx/map/brickl", false, 0xffff, true);
            tiles[BRICK_TL] = new TileInfo("gfx/map/bricktl", false, 0xffff, true);
            tiles[BRICK_TR] = new TileInfo("gfx/map/bricktr", false, 0xffff, true);
            tiles[BRICK_BL] = new TileInfo("gfx/map/brickbl", false, 0xffff, true);
            tiles[BRICK_BR] = new TileInfo("gfx/map/brickbr", false, 0xffff, true);
            tiles[BRICK_TLI] = new TileInfo("gfx/map/bricktli", false, 0xffff, true);
            tiles[BRICK_TRI] = new TileInfo("gfx/map/bricktri", false, 0xffff, true);
            tiles[BRICK_BLI] = new TileInfo("gfx/map/brickbli", false, 0xffff, true);
            tiles[BRICK_BRI] = new TileInfo("gfx/map/brickbri", false, 0xffff, true);
            tiles[BRICK_MINI] = new TileInfo("gfx/map/minibrick", false, 0xffff, true);
            tiles[SPACE_MINI] = new TileInfo("gfx/map/minispace", true, 0x0000, false);

            tiles[16] = new TileInfo("gfx/map/brick", false, 0xffff, true);  // Spare slot

            tiles[17] = new TileInfo("gfx/map/brick16", true, 0xfec8, true);
            tiles[18] = new TileInfo("gfx/map/brick17", true, 0xf731, true);
            tiles[19] = new TileInfo("gfx/map/brick18", true, 0x8cef, true);
            tiles[20] = new TileInfo("gfx/map/brick19", true, 0x137f, true);

            tiles[21] = new TileInfo("gfx/map/brick", false, 0xffff, true);  // Spare slot
            tiles[22] = new TileInfo("gfx/map/brick", false, 0xffff, true);  // Spare slot

            for (int i = 22; i < 36; i++)
            {
                int collision = 0x0000;

                if (((i - 21) & 1) != 0) collision |= 0x0033;
                if (((i - 21) & 2) != 0) collision |= 0x00cc;
                if (((i - 21) & 4) != 0) collision |= 0x3300;
                if (((i - 21) & 8) != 0) collision |= 0xcc00;

                tiles[1 + i] = new TileInfo("gfx/map/brick" + i.ToString("00"), true, collision, true);
            }

            tiles[37] = new TileInfo("gfx/map/black", false, 0xffff, true);

            for (int i = 0; i < LAVA_FRAMES; i++)
                tiles[LAVA_0 + i] = new TileInfo("gfx/map/lava" + i.ToString("00"), false, 0x0000, false);
            for (int i = 0; i < LAVA_FRAMES; i++)
                tiles[LAVATOP_0 + i] = new TileInfo("gfx/map/lavatop" + i.ToString("00"), true, 0x0000, false);

            tiles[SPIKE] = new TileInfo("gfx/map/spike", true, 0x0000, false);
            tiles[CLEAR_SOLID] = new TileInfo(null, true, 0xffff, false);
#if EDITOR_MODE
            tiles[CAGED_GEM] = new TileInfo("gfx/map/cagedgem_editor", true, 0xffff, false);
            tiles[UPDRAFT] = new TileInfo("gfx/map/dust", true, 0x0000, false);
#else
            tiles[CAGED_GEM] = new TileInfo("gfx/map/cagedgem", true, 0xffff, false);
            tiles[UPDRAFT] = new TileInfo(null, true, 0x0000, false);
#endif
            tiles[CARD_SPOT] = new TileInfo("gfx/map/cardspottl", true, 0x0000, false);
            tiles[CARD_SPOT + 1] = new TileInfo("gfx/map/cardspottr", true, 0x0000, false);
            tiles[CARD_SPOT + 2] = new TileInfo("gfx/map/cardspotbl", true, 0x0000, false);
            tiles[CARD_SPOT + 3] = new TileInfo("gfx/map/cardspotbr", true, 0x0000, false);
        }
            
        public static void LoadGraphicsContent(ContentManager content)
        {
            foreach (TileInfo tile in tiles)
                tile.LoadGraphicsContent(content);
        }

#if EDITOR_MODE
        public static void Draw(SpriteBatch masked, SpriteBatch unmasked, int hover)
        {
            Vector2 pos = new Vector2(0.0f, Global.MENU_HEIGHT);

            for (int i = 0; i < TILE_COUNT; i++)
            {
                if (tiles[i].Tex != null)
                {
                    Color c = Color.White;
                    if (i == hover) c = Color.Red;

                    if (tiles[i].Masked)
                        masked.Draw(tiles[i].Tex, pos, c);
                    else
                        unmasked.Draw(tiles[i].Tex, pos, c);
                }

                pos.X += TileInfo.SIZE;
                if (pos.X > Global.screenWidth - TileInfo.SIZE)
                {
                    pos.X = 0.0f;
                    pos.Y += TileInfo.SIZE;
                }
            }
        }
        
        public static UInt16 FindTexAtPos(Vector2 scrPos)
        {
            int tilesAcross = Global.screenWidth / TileInfo.SIZE;
            int id;

            if (scrPos.Y < Global.MENU_HEIGHT || scrPos.X < 0.0f || scrPos.X >= tilesAcross * TileInfo.SIZE)
                return 0;

            id = (((int)scrPos.Y - Global.MENU_HEIGHT) / TileInfo.SIZE) * tilesAcross;
            id += ((int)scrPos.X / TileInfo.SIZE);

            if (id >= TILE_COUNT)
                return 0;
            else
                return (UInt16)id;
        }

        public static bool Brick(int id)
        {
            return tiles[id].Brick;
        }
#endif

        // Indexer
        public static TileInfo Tile(long index)   // long is a 64-bit integer
        {
            // Read one byte at offset index and return it.
            return tiles[index];
        }
    }
}
